# -*- coding: utf-8 -*-
"""
Simula el movimiento oscilatorio de una masa que
pende de un muelle, sin considerar rozamientos

Se usan unidades del sistema internacional:
    kg para la masa
    m para la altura

El cero de altura se considera en el punto en el
que el muelle está en reposo

El sentido de la fuerza y altura es positivo hacia arriba

Los datos principales son:
    alt : altura actual
    vel: velocidad actual de la masa, en m/s
         inicialmente es cero
    masa : masa del objeto
    tiempo : tiempo transcurrido

@author: Michael
@date  : ene-2019
"""

from typing import Final

import matplotlib.pyplot as plt

# Constantes:
#    CTE_ELASTICA: Constante elástica del muelle, en N/m
#    G: Gravedad terrestre, en m/s^2
CTE_ELASTICA: Final[float] = 0.52
G: Final[float] = 9.8


def fuerza_muelle(alt: float) -> float:
    """
    Calcula la fuerza que ejerce el muelle
    dada la altura de su extremo

    Args:
        alt: la altura del extremo del muelle, en m
    Returns:
        la fuerza que ejerce el muelle, en N
    """

    # retornamos la fuerza calculada por medio de la ley de Hooke
    return -CTE_ELASTICA*alt


def avanza_tiempo(alt: float, vel: float, masa: float,
                  tiempo: float) -> tuple[float, float]:
    """
    Calcula la nueva altura y velocidad de la masa,
    transcurrido un tiempo especificado

    El tiempo debe ser pequeño, para poder considerar la
    aceleración constante en ese intervalo

    Args:
        alt: la altura actual del extremo del muelle, en m
        vel: la velocidad actual del extremo del muelle, m/s
        masa: la masa del cuerpo que pende del muelle, en kg
        tiempo: el tiempo transcurrido, en s
    Returns:
        la nueva altura y velocidad de la masa,
        después de transcurrido un tiempo t
    """

    # Calculamos la fuerza aplicada sobre la masa, en N
    fuerza: float = fuerza_muelle(alt) - masa*G
    # aceleración por la ley de newton
    aceleracion: float = fuerza/masa
    # ecuacion del movimiento uniformemente acelerado
    return (alt + vel*tiempo + (aceleracion*tiempo**2)/2,
            vel + aceleracion*tiempo)


def main():
    """
    Simula el movimiento de una masa suspendida de un
    muelle durante un tiempo, y pone en pantalla
    altura y velocidad. Unidades del S.I.
    """
    masa: float = 0.25
    incremento: float = 0.01
    alt: float = 0.2
    vel: float = 0

    # Avanzar la simulación y mostrar resultados
    print(f"Altura={alt} m. Velocidad={vel} m/s")
    alt, vel = avanza_tiempo(alt, vel, masa, incremento)
    print(f"Altura={alt} m. Velocidad={vel} m/s")
    alt, vel = avanza_tiempo(alt, vel, masa, incremento)
    print(f"Altura={alt} m. Velocidad={vel} m/s")


def main2():
    """
    Simula el movimiento de una masa suspendida de un
    muelle durante tiempo, y pone en pantalla
    altura y velocidad. Unidades del S.I.
    """
    masa: float = 0.25
    incremento: float = 0.01
    alt: float = 0.2
    vel: float = 0

    # Avanzar la simulación y mostrar resultados
    for _ in range(10):
        print(f"Altura={alt} m. Velocidad={vel} m/s")
        (alt, vel) = avanza_tiempo(alt, vel, masa, incremento)


def main3():
    """
    Simula el movimiento de una masa suspendida de un
    muelle durante 20 segundos, y dibuja en pantalla la
    trayectoria
    """
    masa: float = 0.25
    tiempo: float = 0
    incremento: float = 0.001
    alt: float = 10
    vel: float = 0

    # Avanzar la simulación y mostrar resultados

    # creamos dos listas para los puntos de la piedra p
    list_x = [tiempo]
    list_y = [alt]

    # avanzamos el tiempo y metemos las nuevas posiciones en las listas
    for _ in range(20000):
        alt, vel = avanza_tiempo(alt, vel, masa, incremento)
        tiempo += incremento
        list_x.append(tiempo)
        list_y.append(alt)

    # Creamos la grafica para dibujar las trayectorias
    plt.plot(list_x, list_y)
    plt.ylabel("Altura (m)")
    plt.xlabel("Tiempo(s)")
    plt.title("Trayectoria de una masa suspendida de un muelle")
    plt.grid(True)
    plt.show()
