# -*- coding: utf-8 -*-
"""
Simula el movimiento de una esfera en un plano inclinado

Se entiende que la esfera rueda sin deslizarse
Utilizamos unidades del sistema internacional (kg, m, s, J)
Para el ángulo utilizamos grados
Ecuaciones del movimiento en : http://www.sc.ehu.es/sbweb/fisica/

@author: Michael
"""
from typing import Final
import math
import matplotlib.pyplot as plt
import numpy

# Constantes
#   G : Gravedad

G: Final[float] = 9.8  # metros/seg**2


def aceleracion(masa: float, radio: float,
                angulo: float) -> float:
    """
    Calcula la aceleración lineal del objeto (m/s)
    Args:
        masa: masa de la esfera, en kg
        radio: radio de la esfera, en m
        angulo: angulo del plano inclinado, en grados
    Returns:
        la aceleración de la esfera
    """

    momento_inercia: float = 2.0*masa*radio**2/5.0
    return (G*math.sin(math.radians(angulo)) /
            (1+momento_inercia/(masa*radio**2)))


def distancia(masa: float, radio: float, angulo: float,
              tiempo: float) -> float:
    """
     Calcula la distancia recorrida por el
     objeto en el tiempo indicado

    Args:
        masa: masa de la esfera, en kg
        radio: radio de la esfera, en m
        angulo: angulo del plano inclinado, en grados
        tiempo transcurrido, en s
    Returns:
        la distancia recorrida por la esfera
    """

    return aceleracion(masa, radio, angulo)*tiempo**2/2.0


def e_cinetica_tras(masa: float, radio: float, angulo: float,
                    tiempo: float) -> float:
    """
    Calcula la energía cinética de traslación,
    del objeto transcurrido el tiempo indicado (Julios)

    Args:
        masa: masa de la esfera, en kg
        radio: radio de la esfera, en m
        angulo: angulo del plano inclinado, en grados
        tiempo transcurrido, en s
    Returns:
        la energía cinética de traslación de la esfera en J
    """

    vel: float = aceleracion(masa, radio, angulo)*tiempo
    return masa*vel**2/2.0


def e_cinetica_rot(masa: float, radio: float, angulo: float,
                   tiempo: float) -> float:
    """
    Calcula la energía cinética de rotación,
    del objeto transcurrido el tiempo indicado (Julios)

    Args:
        masa: masa de la esfera, en kg
        radio: radio de la esfera, en m
        angulo: angulo del plano inclinado, en grados
        tiempo transcurrido, en s
    Returns:
        la energía cinética de rotación de la esfera en J
    """

    # Observar que el cálculo del momento de inercia aparece
    # repetido. Deberíamos implementarlo con una función

    momento_inercia: float = 2.0*masa*radio**2/5.0
    vel_angular: float = (aceleracion(masa, radio, angulo) *
                          tiempo/radio)
    return momento_inercia*vel_angular**2/2.0


def main():
    """
    Programa principal que muestra para una esfera en
    un plano inclinado la distancia y energías a los
    cuatro segundos
    """

    # Crear el sistema plano-esfera
    # Ángulo 30 grados, esfera de 1.5 kg y r=0.2 m
    masa: float = 1.5
    radio: float = 0.2
    angulo: float = 30

    # Mostrar resultados

    print("Dist. a los 4 seg: " +
          f"{distancia(masa, radio, angulo, 4.0):.3f} m")
    print("E. C. tras. a 4 seg: " +
          f"{e_cinetica_tras(masa, radio, angulo, 4.0):.3f} J")
    print("E. C. rot a 4 seg: " +
          f"{e_cinetica_rot(masa, radio, angulo, 4.0):.3f} J")


def simula():
    """
    Programa principal que simula para una esfera en
    un plano inclinado las energías en función de las
    distancias recorridas
    """

    # Crear el sistema plano-esfera
    # Ángulo 30 grados, esfera de 1.5 kg y r=0.2 m
    masa: float = 1.5
    radio: float = 0.2
    angulo: float = 30

    # Creamos unas listas vacías para distancias y energías
    list_dist = []
    list_e_tras = []
    list_e_rot = []

    # Mostrar simulación de las energías entre 0 y 4 seg
    for tiempo in numpy.linspace(0.0, 4.0, num=100):
        list_dist.append(distancia(masa, radio, angulo, tiempo))
        list_e_tras.append(e_cinetica_tras(masa, radio, angulo,
                                           tiempo))
        list_e_rot.append(e_cinetica_rot(masa, radio, angulo, tiempo))

    # Creamos la grafica para dibujar las trayectorias
    plt.plot(list_dist, list_e_tras, "r", label="Enegía traslación")
    plt.plot(list_dist, list_e_rot, "b", label="Energía rotación")
    plt.ylabel("Energía (J)")
    plt.xlabel("Dist(m)")
    plt.title("Energía cinética de una esfera en un plano inclinado")
    plt.grid(True)
    plt.legend()
    plt.show()
